@@ -30,7 +30,7 @@ import sys
 import signal
 import os
 import gzip
-import numpy as np
+import numpy as np 
 
 try:
     import pyqtgraph as pg
@@ -54,7 +54,7 @@ import h5py
 import time
 from progressbar import ProgressBar
 import asyncio
-from pymanip.asynctools import synchronize_function
+from pymanip.asynctools import synchronize_function, synchronize_generator
 
 
 class CameraTimeout(Exception):
@@ -64,7 +64,8 @@ class CameraTimeout(Exception):
     pass
 
 
-def save_image(im, ii, basename, zerofill, file_format, compression, compression_level):
+def save_image(im, ii, basename, zerofill, file_format, compression, compression_level,
+               color_order=None):
     """This function is a simple general function to save an input image from the camera
     to disk.
 
@@ -99,6 +100,8 @@ def save_image(im, ii, basename, zerofill, file_format, compression, compression
         with h5py.File(filename, "w") as f:
             f.attrs["counter"] = im.metadata["counter"]
             f.attrs["timestamp"] = im.metadata["timestamp"].timestamp()
+            if color_order is not None:
+                f.attrs["color_order"] = color_order
             # compression='gzip' trop lent pour 30 fps
             # compression='lzf' presque bon mais un peu lent à 30 fps
             f.create_dataset("image", data=im, compression=compression)
@@ -110,6 +113,8 @@ def save_image(im, ii, basename, zerofill, file_format, compression, compression
             params = (cv2.IMWRITE_PNG_COMPRESSION, compression_level)
         else:
             params = None
+        if color_order == "RGB":
+            im = cv2.cvtColor(im, cv2.COLOR_RGB2BGR)
         cv2.imwrite(filename, im, params)
 
 
@@ -204,31 +209,6 @@ class Camera:
         be overriden for the next frame, and which is no longer defined when the generator object is cleaned up.
         The users are responsible for copying the array, if they want a persistant copy.
 
-        The typical code structure in sub-classes must be like this:
-
-        .. code-block:: python
-
-            def acquisition(self, *args, **kwargs):
-
-                # ... setup code ...
-
-                while count < num:
-
-                    # ... grab frame code ...
-
-                    stop_signal = yield MetadataArray(frame,
-                                                      metadata={"counter": count,
-                                                                "timestamp": timestamp,
-                                                                }
-                                                      )
-                    if stop_signal:
-                        break
-
-                # ... cleanup code ...
-
-                if stop_signal:
-                    yield True
-
         User-level code will use the generator in this manner:
 
         .. code-block:: python
@@ -245,7 +225,9 @@ class Camera:
                     # no need to break here because the gen will be automatically exhausted
 
         """
-        raise NotImplementedError()
+        yield from synchronize_generator(
+            self.acquisition_async, num, timeout, raw, None, raise_on_timeout
+        )
 
     async def acquisition_async(
         self,
@@ -361,14 +343,30 @@ class Camera:
                     im = cv2.rotate(im, cv2.ROTATE_90_COUNTERCLOCKWISE)
                 elif rotate == -90.0:
                     im = cv2.rotate(im, cv2.ROTATE_90_CLOCKWISE)
+                try:
+                    l, c = im.shape
+                    color = False
+                except ValueError:
+                    l, c, ncomp = im.shape
+                    color = True
+
                 if slice_:
-                    img = (maxint // (maximum - minimum)) * (
-                        im[slice_[0] : slice_[1], slice_[2] : slice_[3]] - minimum
-                    )
+                    if color:
+                        img = (maxint // (maximum - minimum)) * (
+                            im[slice_[0] : slice_[1], slice_[2] : slice_[3], :] - minimum
+                        )
+                    else:
+                        img = (maxint // (maximum - minimum)) * (
+                            im[slice_[0] : slice_[1], slice_[2] : slice_[3]] - minimum
+                        )
+
                 else:
                     img = (maxint // (maximum - minimum)) * (im - minimum)
-                l, c = img.shape
-                cv2.imshow(name, cv2.resize(img, (int(c * zoom), int(l * zoom))))
+                img = cv2.resize(img, (int(c * zoom), int(l * zoom)))
+                if color and self.color_order == "RGB":
+                    # OpenCV works in BGR order.
+                    img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
+                cv2.imshow(name, img)
                 k = cv2.waitKey(1)
                 if k in (0x1B, ord("s")):
                     clean = await preview_generator.asend(True)
@@ -494,7 +492,7 @@ class Camera:
 
         # instantiate generator
         if not hasattr(self, "preview_generator"):
-            self.preview_generator = self.acquisition(timeout=5, raise_on_timeout=False)
+            self.preview_generator = self.acquisition(timeout=1000, raise_on_timeout=False)
 
         if just_started:
             self.bkgrd = None
@@ -504,6 +502,14 @@ class Camera:
         # for camera reading)
         img = next(self.preview_generator)
         if img is not None:
+            try:
+                l, c = img.shape
+                color = False
+            except ValueError:
+                l, c, ncomp = img.shape
+                color = True
+            if zoom != 1.0:
+                img = cv2.resize(img, (int(c * zoom), int(l * zoom)))
             if rotate == 90.0:
                 img = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
             elif rotate == -90.0:
@@ -520,9 +526,16 @@ class Camera:
                 img = img - self.bkgrd  # self.bkgrd - img
                 img[img < 0] = 0
                 img = img.astype(np.uint16)
-
+            if not color:
+                img = img.T
+            else:
+                img = np.transpose(img, axes=(1,0,2))
+                if self.color_order == "BGR":
+                    # Qt comme Matplotlib travaille en RGB.
+                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+            
             self.image_view.setImage(
-                img.T, autoRange=False, autoLevels=False, autoHistogramRange=False
+                img, autoRange=False, autoLevels=False, autoHistogramRange=False
             )
             if not self.range_set:
                 self.image_view.autoRange()
@@ -651,24 +664,30 @@ class Camera:
         )
         self.acqinterrupted = False
         async for im in acqgen:
-            if dryrun:
-                continue
             if ii == 0:
                 print(im.dtype)
-            if delay_save:
-                images.append(im.copy())
-            else:
-                start_time = time.process_time()
-                save_image(
-                    im,
-                    ii,
-                    basename,
-                    zerofill,
-                    file_format,
-                    compression,
-                    compression_level,
-                )
-                computation_time += time.process_time() - start_time
+            if not dryrun:
+                if delay_save:
+                    images.append(im.copy())
+                else:
+                    start_time = time.process_time()
+                    if hasattr(self, "color_order"):
+                        color_order = self.color_order
+                    else:
+                        color_order = None
+                    if im.ndim < 3:
+                        color_order = None
+                    save_image(
+                        im,
+                        ii,
+                        basename,
+                        zerofill,
+                        file_format,
+                        compression,
+                        compression_level,
+                        color_order=color_order,
+                    )
+                    computation_time += time.process_time() - start_time
             if hasattr(im, "metadata"):
                 count.append(im.metadata["counter"])
                 ts = im.metadata["timestamp"]
